@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2018 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 %
 % This program is covered by multiple licenses, which are described in
@@ -39,7 +39,9 @@
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/color.h"
+#include "magick/color_lookup.h"
 #include "magick/colormap.h"
+#include "magick/enum_strings.h"
 #include "magick/magick.h"
 #include "magick/monitor.h"
 #include "magick/pixel_cache.h"
@@ -123,13 +125,28 @@ static MagickBool IsMPC(const unsigned char *magick,const size_t length)
 %
 %
 */
+
+#define ThrowMPCReaderException(code_,reason_,image_) \
+do { \
+  if (number_of_profiles > 0) \
+    { \
+      unsigned int _index; \
+      for (_index=0; _index < number_of_profiles; _index++) \
+        { \
+          MagickFreeMemory(profiles[_index].name); \
+          MagickFreeMemory(profiles[_index].info); \
+        } \
+      MagickFreeMemory(profiles); \
+      number_of_profiles=0; \
+    } \
+  ThrowReaderException(code_,reason_,image_); \
+} while (0);
 static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   char
     cache_filename[MaxTextExtent],
     id[MaxTextExtent],
-    keyword[MaxTextExtent],
-    *values;
+    keyword[MaxTextExtent];
 
   ExtendedSignedIntegralType
     offset;
@@ -146,9 +163,6 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   register unsigned char
     *p;
 
-  size_t
-    length;
-
   unsigned int
     status;
 
@@ -169,6 +183,16 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickSignature);
   image=AllocateImage(image_info);
+  /*
+    We must unset the grayscale and monochrome flags by default since
+    the MPC format does not necessarily update the pixel cache while
+    it is read.
+  */
+  image->is_grayscale=MagickFalse;
+  image->is_monochrome=MagickFalse;
+  /*
+    Open blob
+  */
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFail)
     ThrowReaderException(FileOpenError,UnableToOpenFile,image);
@@ -187,11 +211,10 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     /*
       Decode image header;  header terminates one character beyond a ':'.
     */
-    length=MaxTextExtent;
-    values=AllocateString((char *) NULL);
     quantum_depth=QuantumDepth;
     image->depth=8;
     image->compression=NoCompression;
+    image->storage_class=DirectClass;
     while (isgraph(c) && (c != ':'))
     {
       register char
@@ -202,22 +225,28 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           char
             *comment;
 
+          size_t
+            comment_length;
+
           /*
             Read comment-- any text between { }.
           */
-          length=MaxTextExtent;
-          comment=AllocateString((char *) NULL);
+          comment_length=MaxTextExtent;
+          comment=MagickAllocateMemory(char *,comment_length);
+          if (comment == (char *) NULL)
+            ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
+              image);
           p=comment;
           for ( ; comment != (char *) NULL; p++)
           {
             c=ReadBlobByte(image);
             if ((c == EOF) || (c == '}'))
               break;
-            if ((unsigned long) (p-comment+1) >= length)
+            if ((size_t) (p-comment+1) >= comment_length)
               {
                 *p='\0';
-                length<<=1;
-                MagickReallocMemory(char *,comment,length);
+                comment_length<<=1;
+                MagickReallocMemory(char *,comment,comment_length);
                 if (comment == (char *) NULL)
                   break;
                 p=comment+strlen(comment);
@@ -225,7 +254,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             *p=c;
           }
           if (comment == (char *) NULL)
-            ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+            ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
               image);
           *p='\0';
           (void) SetImageAttribute(image,"comment",comment);
@@ -235,40 +264,71 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
       else
         if (isalnum(c))
           {
+            char
+              *values;
+
+            size_t
+              values_length;
+
+            MagickBool
+              in_brace=MagickFalse;
+
             /*
-              Determine a keyword and its value.
+              Get keyword.
             */
+            keyword[0]='\0';
             p=keyword;
             do
             {
               if ((p-keyword) < (MaxTextExtent-1))
                 *p++=c;
               c=ReadBlobByte(image);
-            } while (isalnum(c) || (c == '-'));
+            } while ((c != '=') && (c != EOF));
             *p='\0';
-            while (isspace(c) || (c == '='))
+            if (c == EOF)
+              ThrowMPCReaderException(CorruptImageWarning,ImproperImageHeader,image);
+
+            /*
+              Get values.
+
+              Values not containing spaces are terminated by the first
+              white-space (or new-line) enountered.  Values containing
+              spaces and/or new-lines must be surrounded by braces.
+            */
+            values_length=MaxTextExtent;
+            values=MagickAllocateMemory(char *,values_length);
+            if (values == (char *) NULL)
+              ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+            values[0]='\0';
+            c=ReadBlobByte(image);
+            in_brace=(c == '{');
+            if (in_brace)
               c=ReadBlobByte(image);
             p=values;
-            while ((c != '}') && (c != EOF))
-            {
-              if ((p-values+1) >= (int) length)
-                {
-                  *p='\0';
-                  length<<=1;
-                  MagickReallocMemory(char *,values,length);
-                  if (values == (char *) NULL)
+            while ((((!in_brace) && (c != '\n')) ||
+                     ((in_brace) && (c != '}'))) &&
+                   (c != EOF))
+              {
+                if ((size_t) (p-values+1) >= values_length)
+                  {
+                    *p='\0';
+                    values_length<<=1;
+                    MagickReallocMemory(char *,values,values_length);
+                    if (values == (char *) NULL)
+                      break;
+                    p=values+strlen(values);
+                  }
+                if (values == (char *) NULL)
+                  ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                *p++=c;
+                c=ReadBlobByte(image);
+                if (!in_brace)
+                  if (isspace(c))
                     break;
-                  p=values+strlen(values);
-                }
-              if (values == (char *) NULL)
-                ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
-              *p++=c;
-              c=ReadBlobByte(image);
-              if (*values != '{')
-                if (isspace(c))
-                  break;
-            }
+              }
             *p='\0';
+            (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                                  "keyword=\"%s\" values=\"%s\"",keyword,values);
             /*
               Assign a value to the specified keyword.
             */
@@ -315,7 +375,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"colors") == 0)
                   {
-                    image->colors=atol(values);
+                    image->colors=MagickAtoL(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"colorspace") == 0)
@@ -357,7 +417,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"columns") == 0)
                   {
-                    image->columns= atol(values);
+                    image->columns= MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -369,17 +429,17 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"delay") == 0)
                   {
-                    image->delay=atol(values);
+                    image->delay=MagickAtoL(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"depth") == 0)
                   {
-                    image->depth=atol(values);
+                    image->depth=MagickAtoL(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"dispose") == 0)
                   {
-                    image->dispose=(DisposeType) atol(values);
+                    image->dispose=(DisposeType) MagickAtoL(values);
                     if (LocaleCompare(values,"Background") == 0)
                       image->dispose=BackgroundDispose;
                     else
@@ -399,7 +459,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"error") == 0)
                   {
-                    image->error.mean_error_per_pixel=atof(values);
+                    image->error.mean_error_per_pixel=MagickAtoF(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -411,13 +471,13 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"gamma") == 0)
                   {
-                    image->gamma=atof(values);
+                    image->gamma=MagickAtoF(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"grayscale") == 0)
                   {
-                    image->is_grayscale=(LocaleCompare(values,"True") == 0) ||
-                      (LocaleCompare(values,"true") == 0);
+                    if (LocaleCompare(values,"True") == 0)
+                      image->is_grayscale=MagickTrue;
                     break;
                   }
                 if (LocaleCompare(keyword,"green-primary") == 0)
@@ -441,7 +501,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"iterations") == 0)
                   {
-                    image->iterations=atol(values);
+                    image->iterations=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -465,18 +525,18 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"maximum-error") == 0)
                   {
-                    image->error.normalized_maximum_error=atof(values);
+                    image->error.normalized_maximum_error=MagickAtoF(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"mean-error") == 0)
                   {
-                    image->error.normalized_mean_error=atof(values);
+                    image->error.normalized_mean_error=MagickAtoF(values);
                     break;
                   }
                 if (LocaleCompare(keyword,"monochrome") == 0)
                   {
-                    image->is_monochrome=(LocaleCompare(values,"True") == 0) ||
-                      (LocaleCompare(values,"true") == 0);
+                    if (LocaleCompare(values,"True") == 0)
+                      image->is_monochrome=MagickTrue;
                     break;
                   }
                 if (LocaleCompare(keyword,"montage") == 0)
@@ -488,6 +548,24 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   *values == '{' ? values+1 : values);
                 break;
               }
+              case 'o':
+              case 'O':
+              {
+                if (LocaleCompare(keyword,"opaque") == 0)
+                  {
+                    image->matte=(LocaleCompare(values,"True") == 0) ||
+                      (LocaleCompare(values,"true") == 0);
+                    break;
+                  }
+                if (LocaleCompare(keyword,"orientation") == 0)
+                  {
+                    image->orientation=StringToOrientationType(values);
+                    break;
+                  }
+                (void) SetImageAttribute(image,keyword,
+                  *values == '{' ? values+1 : values);
+                break;
+              }
               case 'p':
               case 'P':
               {
@@ -507,9 +585,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                     i=(long) number_of_profiles;
                     MagickReallocMemory(ProfileInfo *,profiles,(i+1)*sizeof(ProfileInfo));
                     if (profiles == (ProfileInfo *) NULL)
-                      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                      {
+                        MagickFreeMemory(values);
+                        ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+                      }
                     profiles[i].name=AllocateString(keyword+8);
-                    profiles[i].length=atol(values);
+                    profiles[i].length=MagickAtoL(values);
                     profiles[i].info=(unsigned char *) NULL;
                     number_of_profiles++;
                     break;
@@ -518,25 +599,12 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   *values == '{' ? values+1 : values);
                 break;
               }
-              case 'o':
-              case 'O':
-              {
-                if (LocaleCompare(keyword,"opaque") == 0)
-                  {
-                    image->matte=(LocaleCompare(values,"True") == 0) ||
-                      (LocaleCompare(values,"true") == 0);
-                    break;
-                   }
-                (void) SetImageAttribute(image,keyword,
-                  *values == '{' ? values+1 : values);
-                break;
-              }
               case 'q':
               case 'Q':
               {
                 if (LocaleCompare(keyword,"quantum-depth") == 0)
                   {
-                    quantum_depth=atol(values);
+                    quantum_depth=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -577,7 +645,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                   }
                 if (LocaleCompare(keyword,"rows") == 0)
                   {
-                    image->rows=atol(values);
+                    image->rows=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -589,7 +657,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 if (LocaleCompare(keyword,"scene") == 0)
                   {
-                    image->scene=atol(values);
+                    image->scene=MagickAtoL(values);
                     break;
                   }
                 (void) SetImageAttribute(image,keyword,
@@ -634,14 +702,25 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
                 break;
               }
             }
+            MagickFreeMemory(values);
           }
         else
-          c=ReadBlobByte(image);
+          {
+            c=ReadBlobByte(image);
+          }
       while (isspace(c))
         c=ReadBlobByte(image);
     }
-    MagickFreeMemory(values);
     (void) ReadBlobByte(image);
+
+    (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+                          "id=\"%s\" class=%s compression=%s matte=%s "
+                          "columns=%lu rows=%lu depth=%u",
+                          id,ClassTypeToString(image->storage_class),
+                          CompressionTypeToString(image->compression),
+                          MagickBoolToString(image->matte),
+                          image->columns, image->rows, image->depth);
+
     /*
       Verify that required image information is defined.
     */
@@ -649,9 +728,9 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         (image->storage_class == UndefinedClass) ||
         (image->compression == UndefinedCompression) || (image->columns == 0) ||
         (image->rows == 0))
-      ThrowReaderException(CorruptImageError,ImproperImageHeader,image);
+      ThrowMPCReaderException(CorruptImageError,ImproperImageHeader,image);
     if (quantum_depth != QuantumDepth)
-      ThrowReaderException(CacheError,InconsistentPersistentCacheDepth,image);
+      ThrowMPCReaderException(CacheError,InconsistentPersistentCacheDepth,image);
     if (image->montage != (char *) NULL)
       {
         register char
@@ -662,7 +741,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
         */
         image->directory=AllocateString((char *) NULL);
         if (image->directory == (char *) NULL)
-          ThrowReaderException(CorruptImageError,UnableToReadImageData,image);
+          ThrowMPCReaderException(CorruptImageError,UnableToReadImageData,image);
         p=image->directory;
         do
         {
@@ -675,7 +754,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
               MagickReallocMemory(char *,image->directory,
                 (strlen(image->directory)+MaxTextExtent+1));
               if (image->directory == (char *) NULL)
-                ThrowReaderException(CorruptImageError,UnableToReadImageData,
+                ThrowMPCReaderException(CorruptImageError,UnableToReadImageData,
                   image);
               p=image->directory+strlen(image->directory);
             }
@@ -695,7 +774,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             continue;
           profiles[i].info=MagickAllocateMemory(unsigned char *,profiles[i].length);
           if (profiles[i].info == (unsigned char *) NULL)
-            ThrowReaderException(CorruptImageError,UnableToReadGenericProfile,
+            ThrowMPCReaderException(CorruptImageError,UnableToReadGenericProfile,
               image);
           (void) ReadBlob(image,profiles[i].length,profiles[i].info);
           (void) SetImageProfile(image,profiles[i].name,profiles[i].info,
@@ -713,7 +792,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
           Create image colormap.
         */
         if (!AllocateImageColormap(image,image->colors))
-          ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+          ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
             image);
         if (image->colors == 0)
           for (i=0; i < 256; i++)
@@ -737,7 +816,7 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
             packet_size=image->depth > 8 ? 6 : 3;
             colormap=MagickAllocateMemory(unsigned char *,packet_size*image->colors);
             if (colormap == (unsigned char *) NULL)
-              ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,
+              ThrowMPCReaderException(ResourceLimitError,MemoryAllocationFailed,
                 image);
             (void) ReadBlob(image,packet_size*image->colors,colormap);
             p=colormap;
@@ -770,12 +849,16 @@ static Image *ReadMPCImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (image_info->ping && (image_info->subrange != 0))
       if (image->scene >= (image_info->subimage+image_info->subrange-1))
         break;
+
+    if (CheckImagePixelLimits(image, exception) != MagickPass)
+      ThrowMPCReaderException(ResourceLimitError,ImagePixelLimitExceeded,image);
+
     /*
       Attach persistent pixel cache.
     */
     status=PersistCache(image,cache_filename,MagickTrue,&offset,exception);
     if (status == MagickFail)
-      ThrowReaderException(CacheError,UnableToPeristPixelCache,image);
+      ThrowMPCReaderException(CacheError,UnableToPeristPixelCache,image);
     /*
       Proceed to next image.
     */
@@ -929,16 +1012,19 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
 
   ImageProfileIterator
     profile_iterator;
-  
+
   const char
     *profile_name;
-  
+
   const unsigned char
     *profile_info;
-  
+
   size_t
     profile_length;
 
+  size_t
+    image_list_length;
+
   /*
     Open persistent cache.
   */
@@ -946,6 +1032,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  image_list_length=GetImageListLength(image);
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFail)
     ThrowWriterException(FileOpenError,UnableToOpenFile,image);
@@ -1138,9 +1225,15 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
           image->chromaticity.white_point.x,image->chromaticity.white_point.y);
         (void) WriteBlobString(image,buffer);
       }
+    if (image->orientation != UndefinedOrientation)
+      {
+        FormatString(buffer,"orientation=%s\n",
+                     OrientationTypeToString(image->orientation));
+        (void) WriteBlobString(image,buffer);
+      }
     /*
       Attached profiles.
-    */    
+    */
     profile_iterator=AllocateImageProfileIterator(image);
     if (profile_iterator)
       {
@@ -1263,7 +1356,7 @@ static MagickPassFail WriteMPCImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+    status=MagickMonitorFormatted(scene++,image_list_length,
                                   &image->exception,SaveImagesText,
                                   image->filename);
     if (status == False)
