@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003-2018 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -81,12 +81,20 @@ static unsigned int
 %
 %
 */
+#define ThrowYUVReaderException(code_,reason_,image_) \
+{ \
+  MagickFreeMemory(scanline);                 \
+  DestroyImage(chroma_image);                 \
+  DestroyImage(resize_image);                 \
+  ThrowReaderException(code_,reason_,image_); \
+}
+
 static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
-    *chroma_image,
+    *chroma_image = (Image *) NULL,
     *image,
-    *resize_image;
+    *resize_image = (Image *) NULL;
 
   long
     horizontal_factor,
@@ -113,7 +121,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     count;
 
   unsigned char
-    *scanline;
+    *scanline = (unsigned char *) NULL;
 
   MagickPassFail
     status;
@@ -131,7 +139,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   status=MagickPass;
   image=AllocateImage(image_info);
   if ((image->columns == 0) || (image->rows == 0))
-    ThrowReaderException(OptionError,MustSpecifyImageSize,image);
+    ThrowYUVReaderException(OptionError,MustSpecifyImageSize,image);
   image->depth=8;
   interlace=image_info->interlace;
   horizontal_factor=2;
@@ -147,7 +155,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         vertical_factor=horizontal_factor;
       if ((horizontal_factor != 1) && (horizontal_factor != 2) &&
           (vertical_factor != 1) && (vertical_factor != 2))
-        ThrowReaderException(OptionError,UnsupportedSamplingFactor,
+        ThrowYUVReaderException(OptionError,UnsupportedSamplingFactor,
           image);
     }
   if ((interlace == UndefinedInterlace) ||
@@ -164,7 +172,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
       */
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
       if (status == MagickFail)
-        ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+        ThrowYUVReaderException(FileOpenError,UnableToOpenFile,image);
       for (i=0; i < image->offset; i++)
         {
           if (EOF == ReadBlobByte(image))
@@ -180,13 +188,13 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
   else
     scanline=MagickAllocateMemory(unsigned char *,image->columns);
   if (scanline == (unsigned char *) NULL)
-    ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+    ThrowYUVReaderException(ResourceLimitError,MemoryAllocationFailed,image);
   do
   {
     chroma_image=CloneImage(image,image->columns/horizontal_factor,
       image->rows/vertical_factor,True,exception);
     if (chroma_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+      ThrowYUVReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     /*
       Convert raster image to pixel packets.
     */
@@ -198,7 +206,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AppendImageFormat("Y",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == False)
-          ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+          ThrowYUVReaderException(FileOpenError,UnableToOpenFile,image);
       }
     for (y=0; y < (long) image->rows; y++)
     {
@@ -243,8 +251,8 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           q=SetImagePixels(image,0,y,image->columns,1);
           if (q == (PixelPacket *) NULL)
             {
-              break;
               status=MagickFail;
+              break;
             }
           for (x=0; x < (long) image->columns; x++)
           {
@@ -267,7 +275,8 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           }
       if (image->previous == (Image *) NULL)
         if (!MagickMonitorFormatted(y,image->rows,exception,LoadImageText,
-                                    image->filename))
+                                    image->filename,
+                                    image->columns,image->rows))
           {
             status=MagickFail;
             break;
@@ -281,7 +290,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         AppendImageFormat("U",image->filename);
         status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
         if (status == False)
-          ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+          ThrowYUVReaderException(FileOpenError,UnableToOpenFile,image);
       }
     if (interlace != NoInterlace)
       {
@@ -316,7 +325,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
           AppendImageFormat("V",image->filename);
           status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
           if (status == False)
-            ThrowReaderException(FileOpenError,UnableToOpenFile,image);
+            ThrowYUVReaderException(FileOpenError,UnableToOpenFile,image);
         }
       for (y=0; y < (long) chroma_image->rows; y++)
       {
@@ -348,8 +357,9 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
     resize_image=ResizeImage(chroma_image,image->columns,image->rows,
       TriangleFilter,1.0,exception);
     DestroyImage(chroma_image);
+    chroma_image=(Image *) NULL;
     if (resize_image == (Image *) NULL)
-      ThrowReaderException(ResourceLimitError,MemoryAllocationFailed,image);
+      ThrowYUVReaderException(ResourceLimitError,MemoryAllocationFailed,image);
     for (y=0; y < (long) image->rows; y++)
     {
       q=GetImagePixels(image,0,y,image->columns,1);
@@ -374,6 +384,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         }
     }
     DestroyImage(resize_image);
+    resize_image=(Image *) NULL;
     if (status == MagickFail)
       break;
     image->colorspace=YCbCrColorspace;
@@ -405,6 +416,7 @@ static Image *ReadYUVImage(const ImageInfo *image_info,ExceptionInfo *exception)
         if (image->next == (Image *) NULL)
           {
             DestroyImageList(image);
+            MagickFreeMemory(scanline);
             return((Image *) NULL);
           }
         image=SyncNextImageInList(image);
@@ -543,10 +555,14 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     height,
     width;
 
+  size_t
+    image_list_length;
+
   assert(image_info != (const ImageInfo *) NULL);
   assert(image_info->signature == MagickSignature);
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
+  image_list_length=GetImageListLength(image);
   interlace=image_info->interlace;
   horizontal_factor=2;
   vertical_factor=2;
@@ -625,7 +641,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             &chroma_image->exception);
           if (s == (const PixelPacket *) NULL)
             break;
-          for (x=0; x < (long) yuv_image->columns; x++)
+          for (x=0; x < (long) yuv_image->columns; )
           {
             (void) WriteBlobByte(image,ScaleQuantumToChar(s->green));
             (void) WriteBlobByte(image,ScaleQuantumToChar(p->red));
@@ -635,11 +651,13 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
             p++;
             s++;
             x++;
+            x++;
           }
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+                                          image->columns,image->rows))
                 break;
         }
         DestroyImage(yuv_image);
@@ -663,7 +681,8 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
           if (image->previous == (Image *) NULL)
             if (QuantumTick(y,image->rows))
               if (!MagickMonitorFormatted(y,image->rows,&image->exception,
-                                          SaveImageText,image->filename))
+                                          SaveImageText,image->filename,
+                                          image->columns,image->rows))
                 break;
         }
         DestroyImage(yuv_image);
@@ -722,7 +741,7 @@ static unsigned int WriteYUVImage(const ImageInfo *image_info,Image *image)
     if (image->next == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=MagickMonitorFormatted(scene++,GetImageListLength(image),
+    status=MagickMonitorFormatted(scene++,image_list_length,
                                   &image->exception,SaveImagesText,
                                   image->filename);
     if (status == False)
