@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 GraphicsMagick Group
+% Copyright (C) 2003 - 2018 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -40,7 +40,9 @@
 */
 #include "magick/studio.h"
 #include "magick/alpha_composite.h"
+#include "magick/analyze.h"
 #include "magick/color.h"
+#include "magick/color_lookup.h"
 #include "magick/composite.h"
 #include "magick/constitute.h"
 #include "magick/gem.h"
@@ -53,48 +55,39 @@
 #include "magick/xwindow.h"
 #if defined(HasTTF)
 
-#if defined(__MINGW32__)
-#  undef interface  /* Remove interface define */
-#endif
+#  if defined(__MINGW32__)
+#    undef interface  /* Remove interface define */
+#  endif
 
-#if defined(HAVE_FT2BUILD_H)
-  /*
-    Modern libwmf2 installs may require that <ft2build.h> be included
-    before including other libwmf headers.  Including <ft2build.h>
-    establishes definitions used by other FreeType headers.
-  */
-#  include <ft2build.h>
-#endif /* defined(HAVE_FT2BUILD_H) */
-
-#if defined(FT_FREETYPE_H)
-#  include FT_FREETYPE_H
-#else
-#  include <freetype/freetype.h>
-#endif /* defined(FT_FREETYPE_H) */
-
-#if defined(FT_GLYPH_H)
-#  include FT_GLYPH_H
-#else
-#  include <freetype/ftglyph.h>
-#endif /* defined(FT_GLYPH_H) */
-
-#if defined(FT_OUTLINE_H)
-#  include FT_OUTLINE_H
-#else
-#  include <freetype/ftoutln.h>
-#endif /* defined(FT_OUTLINE_H) */
-
-#if defined(FT_BBOX_H)
-#  include FT_BBOX_H
-#else
-#  include <freetype/ftbbox.h>
-#endif /* defined(FT_BBOX_H) */
+#  if defined(HAVE_FT2BUILD_H)
+     /*
+       Modern FreeType2 installs require that <ft2build.h> be included
+       before including other FreeType2 headers.  Including
+       <ft2build.h> establishes definitions used by other FreeType2
+       headers.
+     */
+#    include <ft2build.h>
+#    include FT_FREETYPE_H
+#    include FT_GLYPH_H
+#    include FT_OUTLINE_H
+#    include FT_BBOX_H
+#  else
+     /*
+       Very old way to include FreeType2
+     */
+#    include <freetype/freetype.h>
+#    include <freetype/ftglyph.h>
+#    include <freetype/ftoutln.h>
+#    include <freetype/ftbbox.h>
+#  endif /* defined(HAVE_FT2BUILD_H) */
 
 #endif /* defined(HasTTF) */
 
 /*
   Forward declarations.
 */
+typedef magick_int32_t magick_code_point_t;
+
 static unsigned int
   RenderType(Image *,const DrawInfo *,const PointInfo *,TypeMetric *),
   RenderPostscript(Image *,const DrawInfo *,const PointInfo *,TypeMetric *),
@@ -117,28 +110,46 @@ static unsigned int
 %  any of the following bits of information about the image by embedding
 %  the appropriate special characters:
 %
-%    %b   file size in bytes.
-%    %c   comment.
-%    %d   directory in which the image resides.
-%    %e   extension of the image file.
-%    %f   original filename of the image.
-%    %h   height of image.
-%    %i   filename of the image.
-%    %k   number of unique colors.
-%    %l   image label.
-%    %m   image file format.
-%    %n   number of images in a image sequence.
-%    %o   output image filename.
-%    %p   page number of the image.
-%    %q   image depth (8 or 16).
-%    %p   page number of the image.
-%    %q   image depth (8 or 16).
-%    %s   image scene number.
-%    %t   image filename without any extension.
-%    %u   a unique temporary filename.
-%    %w   image width.
-%    %x   x resolution of the image.
-%    %y   y resolution of the image.
+%    %b   file size
+%    %c   comment
+%    %d   directory
+%    %e   filename extension
+%    %f   filename
+%    %g   page dimensions and offsets
+%    %h   height
+%    %i   input filename
+%    %k   number of unique colors
+%    %l   label
+%    %m   magick
+%    %n   number of scenes
+%    %o   output filename
+%    %p   page number
+%    %q   image bit depth
+%    %r   image type description
+%    %s   scene number
+%    %t   top of filename
+%    %w   width
+%    %x   horizontal resolution
+%    %y   vertical resolution
+%    %A   transparency supported
+%    %C   compression type
+%    %D   GIF disposal method
+%    %G   Original width and height
+%    %H   page height
+%    %M   original filename specification
+%    %O   page offset (x,y)
+%    %P   page dimensions (width,height)
+%    %T   time delay (in centi-seconds)
+%    %U   resolution units
+%    %W   page width
+%    %X   page horizontal offset (x)
+%    %Y   page vertical offset (y)
+%    %@   trim bounding box
+%    %[a] named attribute 'a'
+%    %#   signature
+%    \n   newline
+%    \r   carriage return
+%    %%   % (literal)
 %
 %  The format of the AnnotateImage method is:
 %
@@ -191,6 +202,9 @@ MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info
     height,
     number_lines;
 
+  MagickBool
+    metrics_initialized = MagickFalse;
+
   /*
     Translate any embedded format characters (e.g. %f).
   */
@@ -201,7 +215,7 @@ MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info
   if (draw_info->text == (char *) NULL)
     return(MagickFail);
   if (*draw_info->text == '\0')
-    return(MagickFail);
+    return(MagickPass);
   text=TranslateText((ImageInfo *) NULL,image,draw_info->text);
   if (text == (char *) NULL)
     ThrowBinaryException3(ResourceLimitError,MemoryAllocationFailed,
@@ -226,7 +240,7 @@ MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info
   annotate=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
   matte=image->matte;
-  status=True;
+  status=MagickPass;
   for (i=0; textlist[i] != (char *) NULL; i++)
   {
     if (*textlist[i] == '\0')
@@ -235,8 +249,11 @@ MagickExport MagickPassFail AnnotateImage(Image *image,const DrawInfo *draw_info
       Position text relative to image.
     */
     (void) CloneString(&annotate->text,textlist[i]);
-    if ((i == 0) || (annotate->gravity != NorthWestGravity))
-      (void) GetTypeMetrics(image,annotate,&metrics);
+    if ((!metrics_initialized) || (annotate->gravity != NorthWestGravity))
+      {
+        metrics_initialized=MagickTrue;
+        (void) GetTypeMetrics(image,annotate,&metrics);
+      }
     height=(unsigned long) (metrics.ascent-metrics.descent);
     switch (annotate->gravity)
     {
@@ -470,7 +487,7 @@ static int GetOneCharacter(const unsigned char *text,size_t *length)
   return((int) c);
 }
 
-static unsigned short *EncodeSJIS(const char *text,size_t *count)
+static magick_code_point_t *EncodeSJIS(const char *text,size_t *count)
 {
   int
     c;
@@ -478,21 +495,22 @@ static unsigned short *EncodeSJIS(const char *text,size_t *count)
   register const char
     *p;
 
-  register unsigned short
+  register magick_code_point_t
     *q;
 
   size_t
     length;
 
-  unsigned short
+  magick_code_point_t
     *encoding;
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short *) NULL);
-  encoding=MagickAllocateMemory(unsigned short *,
-    (strlen(text)+MaxTextExtent)*sizeof(unsigned short));
-  if (encoding == (unsigned short *) NULL)
+    return((magick_code_point_t *) NULL);
+  encoding=MagickAllocateArray(magick_code_point_t *,
+                               (strlen(text)+MaxTextExtent),
+                               sizeof(magick_code_point_t));
+  if (encoding == (magick_code_point_t *) NULL)
     MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
       UnableToConvertText);
   q=encoding;
@@ -507,7 +525,7 @@ static unsigned short *EncodeSJIS(const char *text,size_t *count)
           *q++=(unsigned char) *p;
         break;
       }
-    *q=(unsigned short) c;
+    *q=(magick_code_point_t) c;
     q++;
   }
   *count=q-encoding;
@@ -543,23 +561,24 @@ static unsigned short *EncodeSJIS(const char *text,size_t *count)
 %
 %
 */
-static unsigned short *EncodeText(const char *text,size_t *count)
+static magick_code_point_t *EncodeText(const char *text,size_t *count)
 {
   register const char
     *p;
 
-  register unsigned short
+  register magick_code_point_t
     *q;
 
-  unsigned short
+  magick_code_point_t
     *encoding;
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short *) NULL);
-  encoding=MagickAllocateMemory(unsigned short *,
-    (strlen(text)+MaxTextExtent)*sizeof(unsigned short));
-  if (encoding == (unsigned short *) NULL)
+    return((magick_code_point_t *) NULL);
+  encoding=MagickAllocateArray(magick_code_point_t *,
+                               (strlen(text)+MaxTextExtent),
+                               sizeof(magick_code_point_t));
+  if (encoding == (magick_code_point_t *) NULL)
     MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
       UnableToConvertText);
   q=encoding;
@@ -599,10 +618,9 @@ static unsigned short *EncodeText(const char *text,size_t *count)
 %
 %
 */
-
-static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
+static int GetUnicodeCharacter(const unsigned char *text,size_t *length)
 {
-  unsigned long
+  unsigned int
     c;
 
   if (*length < 1)
@@ -611,7 +629,7 @@ static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
   if (!(c & 0x80))
     {
       *length=1;
-      return((long) c);
+      return((int) c);
     }
   if ((*length < 2) || ((text[1] & 0xc0) != 0x80))
     {
@@ -623,7 +641,7 @@ static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
       *length=2;
       c=(text[0] & 0x1f) << 6;
       c|=text[1] & 0x3f;
-      return((long) c);
+      return((int) c);
     }
   if ((*length < 3) || ((text[2] & 0xc0) != 0x80))
     {
@@ -636,7 +654,7 @@ static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
       c=(text[0] & 0xf) << 12;
       c|=(text[1] & 0x3f) << 6;
       c|=text[2] & 0x3f;
-      return((long) c);
+      return((int) c);
     }
   if ((*length < 4) || ((c & 0xf8) != 0xf0) || ((text[3] & 0xc0) != 0x80))
     {
@@ -648,10 +666,10 @@ static long GetUnicodeCharacter(const unsigned char *text,size_t *length)
   c|=(text[1] & 0x3f) << 12;
   c|=(text[2] & 0x3f) << 6;
   c|=text[3] & 0x3f;
-  return((long) c);
+  return((int) c);
 }
 
-static unsigned short *EncodeUnicode(const char *text,size_t *count)
+static magick_code_point_t *EncodeUnicode(const char *text,size_t *count)
 {
   int
     c;
@@ -659,21 +677,22 @@ static unsigned short *EncodeUnicode(const char *text,size_t *count)
   register const char
     *p;
 
-  register unsigned short
+  register magick_code_point_t
     *q;
 
   size_t
     length;
 
-  unsigned short
+  magick_code_point_t
     *unicode;
 
   *count=0;
   if ((text == (char *) NULL) || (*text == '\0'))
-    return((unsigned short *) NULL);
-  unicode=MagickAllocateMemory(unsigned short *,
-    (strlen(text)+MaxTextExtent)*sizeof(unsigned short));
-  if (unicode == (unsigned short *) NULL)
+    return((magick_code_point_t *) NULL);
+  unicode=MagickAllocateArray(magick_code_point_t *,
+                              (strlen(text)+MaxTextExtent),
+                              sizeof(magick_code_point_t));
+  if (unicode == (magick_code_point_t *) NULL)
     MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
       UnableToConvertText);
   q=unicode;
@@ -688,7 +707,7 @@ static unsigned short *EncodeUnicode(const char *text,size_t *count)
           *q++=(unsigned char) *p;
         break;
       }
-    *q=(unsigned short) c;
+    *q=(magick_code_point_t) c;
     q++;
   }
   *count=q-unicode;
@@ -760,6 +779,54 @@ MagickExport MagickPassFail GetTypeMetrics(Image *image,const DrawInfo *draw_inf
   return(status);
 }
 
+
+/*
+  Find a single font family name in a comma-separated list; returns a pointer
+  to where next search should start (i.e., to the terminating character), or null
+  if not found.  Trims leading and trailing white space, and surrounding single
+  quotes.
+*/
+static
+char const *FindCommaDelimitedName
+(
+ char const *  pSearchStart, /*start search here*/
+ char const ** ppStart,      /*return pointer to first character in found string*/
+ char const ** ppEnd         /*return pointer to just past last character in found string*/
+ )
+{ /*FindCommaDelimitedName*/
+
+  int c;
+  char const * pStart;
+  char const * pEnd;
+  char const * pNextSearchStart;
+
+  if ( pSearchStart == 0 )
+    return(0);
+
+  for ( pStart = pSearchStart; (c = *pStart) && (isspace(c) || (c == ','));
+        pStart++ );  /*skip leading spaces and commas*/
+  if ( c == '\0' )
+    return(0);  /*didn't find anything!*/
+
+  for ( pEnd = pStart + 1; (c = *pEnd) && (c != ','); pEnd++ ); /*find terminating comma*/
+  pNextSearchStart = pEnd;
+
+  for ( ; isspace(pEnd[-1]); pEnd-- ); /*trim trailing space; we know there is a non-space character there*/
+
+  /* trim off surrounding single quotes */
+  if ((*pStart == '\'') &&  (*pEnd == '\'') && ((pEnd-pStart) >= 3))
+    {
+      pStart++;
+      pEnd--;
+    }
+
+  *ppStart = pStart;
+  *ppEnd = pEnd;
+  return(pNextSearchStart);
+
+} /*FindCommaDelimitedName*/
+
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -795,17 +862,23 @@ MagickExport MagickPassFail GetTypeMetrics(Image *image,const DrawInfo *draw_inf
 %
 */
 static MagickPassFail RenderType(Image *image,const DrawInfo *draw_info,
-  const PointInfo *offset,TypeMetric *metrics)
+                                 const PointInfo *offset,TypeMetric *metrics)
 {
   const TypeInfo
     *type_info;
 
   DrawInfo
-   *clone_info;
+    *clone_info;
 
   MagickPassFail
     status;
 
+  char
+    OneFontFamilyName[2048]; /*special handling only if font name this long or less*/
+
+  char const *
+    pTheFoundFontFamilyName;
+
   type_info=(const TypeInfo *) NULL;
   if (draw_info->font != (char *) NULL)
     {
@@ -818,9 +891,94 @@ static MagickPassFail RenderType(Image *image,const DrawInfo *draw_info,
         if (IsAccessible(draw_info->font))
           return(RenderFreetype(image,draw_info,(char *) NULL,offset,metrics));
     }
+
+  /* draw_info->family may be a comma-separated list of names ... */
+  pTheFoundFontFamilyName = draw_info->family;
   if (type_info == (const TypeInfo *) NULL)
-    type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
-      draw_info->stretch,draw_info->weight,&image->exception);
+    { /*type_info not yet found*/
+
+      /* stay consistent with previous behavior unless font family contains comma(s) */
+      if (draw_info->family == 0 || (strchr(draw_info->family,',') == 0))
+        { /*null ptr, or no commas in string; preserve previous behavior*/
+
+          type_info=GetTypeInfoByFamily(draw_info->family,draw_info->style,
+                                        draw_info->stretch,draw_info->weight,
+                                        &image->exception);
+
+        } /*null ptr, or no commas in string; preserve previous behavior*/
+      else
+        { /*process as font family list*/
+
+          char const * pNext = draw_info->family, * pStart = 0, * pEnd = 0;
+          while ((pNext = FindCommaDelimitedName(pNext,&pStart,&pEnd)) != 0)
+            { /*found a name*/
+
+              unsigned int NameLength = pEnd - pStart;
+              if  ( NameLength >= sizeof(OneFontFamilyName) )
+                continue;
+              memcpy(OneFontFamilyName,pStart,NameLength);
+              OneFontFamilyName[NameLength] = '\0';
+              type_info = GetTypeInfoByFamily(OneFontFamilyName,
+                                              draw_info->style,
+                                              draw_info->stretch,
+                                              draw_info->weight,
+                                              &image->exception);
+              /*do not allow font substitution*/
+              if ( type_info && (LocaleCompare(OneFontFamilyName,
+                                               type_info->family) == 0) )
+                {
+                  pTheFoundFontFamilyName = OneFontFamilyName;
+                  break;
+                }
+
+            } /*found a name*/
+
+        } /*process as font family list*/
+
+    } /*type_info not yet found*/
+
+  /*
+    We may have performed font substitution.  If so (i.e., font family
+    name does not match), try again assuming draw_info->family is
+    actually a font name.  If we get a font name match, that will
+    override the font substitution.
+  */
+  if ((type_info == (const TypeInfo *) NULL)
+      || /*found font family, but ...*/
+      (pTheFoundFontFamilyName &&
+       (LocaleCompare(pTheFoundFontFamilyName,type_info->family) != 0)))
+    {/*either not found, or different font family (probably font substitution)*/
+
+      /* try to match a font name */
+      const TypeInfo *type_info2 = 0;
+      if (((type_info2 = GetTypeInfo(pTheFoundFontFamilyName,
+                                     &image->exception))
+           == (const TypeInfo *) NULL)
+          && (pTheFoundFontFamilyName != 0)
+          && strlen(pTheFoundFontFamilyName) < sizeof(OneFontFamilyName))
+        {/*change ' ' to '-' and try again*/
+
+          /*
+            Change blanks to hyphens (i.e. make it look like a font
+            name vs. font family).  Will only do this for font names
+            sizeof(OneFontFamilyName) long or less.
+          */
+          char FontNameWithHyphens[sizeof(OneFontFamilyName)];
+          char *pWithHyphens = FontNameWithHyphens;
+          char c;
+          const char *pFound;
+          for (pFound = pTheFoundFontFamilyName;
+               (*pWithHyphens = (((c = *pFound) != ' ') ? c : '-'));
+               pFound++, pWithHyphens++);
+          type_info2 = GetTypeInfo(FontNameWithHyphens,&image->exception);
+
+        } /*change ' ' to '-' and try again*/
+
+      if  ( type_info2 != (const TypeInfo *) NULL )
+        type_info = type_info2;
+
+    } /*either not found, or different font family (probably font substitution)*/
+
   if (type_info == (const TypeInfo *) NULL)
     return(RenderPostscript(image,draw_info,offset,metrics));
   clone_info=CloneDrawInfo((ImageInfo *) NULL,draw_info);
@@ -961,7 +1119,7 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
     encoding_type;
 
   FT_Error
-    status;
+    ft_status;
 
   FT_Face
     face;
@@ -1003,7 +1161,7 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
     *p;
 
   size_t
-    length;
+    length = 0;
 
   static FT_Outline_Funcs
     OutlineMethods =
@@ -1015,39 +1173,45 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
       0, 0
     };
 
-  unsigned int
+  MagickBool
     active;
 
-  unsigned short
+  magick_code_point_t
     *text;
 
+  MagickPassFail
+    status=MagickPass;
+
   glyph.image=(FT_Glyph) 0;
   last_glyph.image=(FT_Glyph) 0;
 
   /*
     Initialize Truetype library.
   */
-  status=FT_Init_FreeType(&library);
-  if (status)
+  ft_status=FT_Init_FreeType(&library);
+  if (ft_status)
     ThrowBinaryException(TypeError,UnableToInitializeFreetypeLibrary,
       draw_info->font);
   if (*draw_info->font != '@')
-    status=FT_New_Face(library,draw_info->font,0,&face);
+    ft_status=FT_New_Face(library,draw_info->font,0,&face);
   else
-    status=FT_New_Face(library,draw_info->font+1,0,&face);
-  if (status != 0)
+    ft_status=FT_New_Face(library,draw_info->font+1,0,&face);
+  if (ft_status != 0)
     {
       (void) FT_Done_FreeType(library);
       ThrowBinaryException(TypeError,UnableToReadFont,draw_info->font)
     }
+  /*
+    Select a charmap
+  */
   if (face->num_charmaps != 0)
-    status=FT_Set_Charmap(face,face->charmaps[0]);
+    /* ft_status= */ (void) FT_Set_Charmap(face,face->charmaps[0]);
   encoding_type=ft_encoding_unicode;
-  status=FT_Select_Charmap(face,encoding_type);
-  if (status != 0)
+  ft_status=FT_Select_Charmap(face,encoding_type);
+  if (ft_status != 0)
     {
       encoding_type=ft_encoding_none;
-      status=FT_Select_Charmap(face,encoding_type);
+      /* ft_status= */ (void) FT_Select_Charmap(face,encoding_type);
     }
   if (encoding != (char *) NULL)
     {
@@ -1063,6 +1227,18 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
         encoding_type=ft_encoding_big5;
       if (LocaleCompare(encoding,"GB2312") == 0)
         encoding_type=ft_encoding_gb2312;
+#if defined(ft_encoding_johab)
+      if (LocaleCompare(encoding,"Johab") == 0)
+        encoding_type=ft_encoding_johab;
+#endif
+#if defined(ft_encoding_latin_1)
+      if (LocaleCompare(encoding,"Latin-1") == 0)
+        encoding_type=ft_encoding_latin_1;
+#endif
+#if defined(ft_encoding_latin_2)
+      if (LocaleCompare(encoding,"Latin-2") == 0)
+        encoding_type=ft_encoding_latin_2;
+#endif
       if (LocaleCompare(encoding,"None") == 0)
         encoding_type=ft_encoding_none;
       if (LocaleCompare(encoding,"SJIScode") == 0)
@@ -1073,8 +1249,8 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
         encoding_type=ft_encoding_unicode;
       if (LocaleCompare(encoding,"Wansung") == 0)
         encoding_type=ft_encoding_wansung;
-      status=FT_Select_Charmap(face,encoding_type);
-      if (status != 0)
+      ft_status=FT_Select_Charmap(face,encoding_type);
+      if (ft_status != 0)
         ThrowBinaryException(TypeError,UnrecognizedFontEncoding,encoding);
     }
   /*
@@ -1104,8 +1280,21 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y2=metrics->ascent+metrics->descent;
   metrics->underline_position=face->underline_position/64.0;
   metrics->underline_thickness=face->underline_thickness/64.0;
+
   /*
-    Convert text to 2-byte format as prescribed by the encoding.
+    If the user-provided text string is NULL or empty, then nothing
+    more to do.
+  */
+  if ((draw_info->text == NULL) || (draw_info->text[0] == '\0'))
+    {
+      (void) FT_Done_Face(face);
+      (void) FT_Done_FreeType(library);
+      return status;
+    }
+
+  /*
+    Convert text to 4-byte format (supporting up to 21 code point
+    bits) as prescribed by the encoding.
   */
   switch (encoding_type)
   {
@@ -1139,10 +1328,17 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
       break;
     }
   }
-  if (text == (unsigned short *) NULL)
+  if (text == (magick_code_point_t *) NULL)
     {
       (void) FT_Done_Face(face);
       (void) FT_Done_FreeType(library);
+      (void) LogMagickEvent(AnnotateEvent,GetMagickModule(),
+                            "Text encoding failed: encoding_type=%ld "
+                            "draw_info->encoding=\"%s\" draw_info->text=\"%s\" length=%ld",
+                            (long) encoding_type,
+                            (draw_info->encoding ? draw_info->encoding : "(null)"),
+                            (draw_info->text ? draw_info->text : "(null)"),
+                            (long) length);
       ThrowBinaryException(ResourceLimitError,MemoryAllocationFailed,
         draw_info->font)
     }
@@ -1181,11 +1377,11 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
       }
     glyph.origin=origin;
     glyph.image=0;
-    status=FT_Load_Glyph(face,glyph.id,FT_LOAD_DEFAULT);
-    if (status != False)
+    ft_status=FT_Load_Glyph(face,glyph.id,FT_LOAD_DEFAULT);
+    if (ft_status != False) /* 0 means success */
       continue;
-    status=FT_Get_Glyph(face->glyph,&glyph.image);
-    if (status != False)
+    ft_status=FT_Get_Glyph(face->glyph,&glyph.image);
+    if (ft_status != False) /* 0 means success */
       continue;
 #if 0
     /*
@@ -1225,70 +1421,125 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
     (void) FT_Glyph_Transform(glyph.image,&affine,&glyph.origin);
     if (draw_info->render)
       {
+        status &= ModifyCache(image,&image->exception);
         if ((draw_info->fill.opacity != TransparentOpacity) ||
             (pattern != (Image *) NULL))
           {
             /*
               Rasterize the glyph.
             */
-            status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
+            ft_status=FT_Glyph_To_Bitmap(&glyph.image,ft_render_mode_normal,
               (FT_Vector *) NULL,True);
-            if (status != False)
+            if (ft_status != False)
               continue;
             bitmap=(FT_BitmapGlyph) glyph.image;
             image->storage_class=DirectClass;
-            point.x=offset->x+bitmap->left;
+            if (bitmap->bitmap.pixel_mode == ft_pixel_mode_mono)
+              {
+                point.x=offset->x+(origin.x >> 6);
+              }
+            else
+              {
+                point.x=offset->x+bitmap->left;
+              }
             point.y=offset->y-bitmap->top;
             p=bitmap->bitmap.buffer;
             /* FIXME: OpenMP */
             for (y=0; y < (long) bitmap->bitmap.rows; y++)
             {
+              int pc = y * bitmap->bitmap.pitch;
+              int pcr = pc;
               if ((ceil(point.y+y-0.5) < 0) ||
                   (ceil(point.y+y-0.5) >= image->rows))
                 {
-                  p+=bitmap->bitmap.width;
                   continue;
                 }
+              /*
+                Try to get whole span.  May fail.
+              */
               q=GetImagePixels(image,(long) ceil(point.x-0.5),
                 (long) ceil(point.y+y-0.5),bitmap->bitmap.width,1);
               active=q != (PixelPacket *) NULL;
-              for (x=0; x < (long) bitmap->bitmap.width; x++)
-              {
-                if (((long) ceil(point.x+x-0.5) < 0) || (*p == 0) ||
-                    ((unsigned long) ceil(point.x+x-0.5) >= image->columns))
-                  {
-                    p++;
-                    q++;
-                    continue;
-                  }
-                if (draw_info->text_antialias)
-                  opacity=ScaleCharToQuantum(*p);
-                else
-                  opacity=((*p) < 127 ? OpaqueOpacity : TransparentOpacity);
-                fill_color=draw_info->fill;
-                if (pattern != (Image *) NULL)
-                  (void) AcquireOnePixelByReference(pattern,&fill_color,
-                    (long) (point.x+x-pattern->tile_info.x) % pattern->columns,
-                    (long) (point.y+y-pattern->tile_info.y) % pattern->rows,
-                            &image->exception);
-                opacity=((MaxRGB-opacity)*(MaxRGB-fill_color.opacity))/MaxRGB;
-                if (!active)
-                  q=GetImagePixels(image,(long) ceil(point.x+x-0.5),
-                    (long) ceil(point.y+y-0.5),1,1);
-                if (q == (PixelPacket *) NULL)
-                  {
-                    p++;
-                    q++;
-                    continue;
-                  }
-                AlphaCompositePixel(q,&fill_color,opacity,q,
-                                    image->matte ? q->opacity : OpaqueOpacity);
-                if (!active)
-                  (void) SyncImagePixels(image);
-                p++;
-                q++;
-              }
-              (void) SyncImagePixels(image);
+              for (x=0; x < (long) bitmap->bitmap.width; x++, pc++)
+                {
+                  if (((long) ceil(point.x+x-0.5) < 0) ||
+                      ((unsigned long) ceil(point.x+x-0.5) >= image->columns))
+                    {
+                      if (active)
+                        q++;
+                      continue;
+                    }
+                  /* 8-bit gray-level pixmap */
+                  if (bitmap->bitmap.pixel_mode == ft_pixel_mode_grays)
+                    {
+                      if (draw_info->text_antialias)
+                        opacity=ScaleCharToQuantum(p[pc]);
+                      else
+                        opacity=(p[pc] < 127 ? OpaqueOpacity : TransparentOpacity);
+                    }
+                  /* 1-bit monochrome bitmap */
+                  else if (bitmap->bitmap.pixel_mode == ft_pixel_mode_mono)
+                    {
+                      opacity=((p[(x >> 3) + pcr] & (1 << (~x & 0x07))) ?
+                               TransparentOpacity : OpaqueOpacity);
+                    }
+                  else
+                    {
+                      continue; /* ignore it? */
+                    }
+                  fill_color=draw_info->fill;
+                  if (pattern != (Image *) NULL)
+                    {
+                      if (AcquireOnePixelByReference
+                          (pattern,&fill_color,
+                           (long) (point.x+x-pattern->tile_info.x) % pattern->columns,
+                           (long) (point.y+y-pattern->tile_info.y) % pattern->rows,
+                           &image->exception) == MagickFail)
+                        {
+                          status=MagickFail;
+                        }
+                    }
+                  /*
+                    If not full span, then get one pixel.
+                  */
+                  if (!active)
+                    q=GetImagePixels(image,(long) ceil(point.x+x-0.5),
+                                     (long) ceil(point.y+y-0.5),1,1);
+                  if (q == (PixelPacket *) NULL)
+                    {
+                      continue;
+                    }
+                  /*
+                    At this point, opacity is 0==transparent to MaxRGB==opaque, and represents an
+                    anti-aliasing edge blending value.  The computation below integrates in the
+                    fill color opacity, and converts the result to 0=opaque to MaxRGB=transparent.
+                  */
+                  opacity=MaxRGB-(opacity*(MaxRGB-fill_color.opacity)+/*round*/(MaxRGB>>1))/MaxRGB;
+                  AlphaCompositePixel(q,&fill_color,opacity,q,
+                                      image->matte ? q->opacity : OpaqueOpacity);
+                  if (!active)
+                    {
+                      /*
+                        Sync the one pixel
+                      */
+                      if (SyncImagePixels(image) != MagickPass)
+                        status=MagickFail;
+                    }
+                  else
+                    {
+                      q++;
+                    }
+                  if (status == MagickFail)
+                    break;
+                }
+              /*
+                Sync the full span
+              */
+              if (active)
+                if (SyncImagePixels(image) != MagickPass)
+                  status=MagickFail;
+              if (status == MagickFail)
+                break;
             }
           }
       }
@@ -1304,7 +1555,7 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
   metrics->bounds.y1/=64.0;
   metrics->bounds.x2/=64.0;
   metrics->bounds.y2/=64.0;
-  if (draw_info->render)
+  if ((status != MagickFail)&& (draw_info->render))
     if ((draw_info->stroke.opacity != TransparentOpacity) ||
         (draw_info->stroke_pattern != (Image *) NULL))
       {
@@ -1325,15 +1576,18 @@ static MagickPassFail RenderFreetype(Image *image,const DrawInfo *draw_info,
   DestroyDrawInfo(clone_info);
   (void) FT_Done_Face(face);
   (void) FT_Done_FreeType(library);
-  return(MagickPass);
+  return(status);
 }
 #else
 static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
-  const char * ARGUNUSED(encoding),const PointInfo * ARGUNUSED(offset),
-  TypeMetric * ARGUNUSED(metrics))
+  const char *encoding,const PointInfo *offset,
+  TypeMetric *metrics)
 {
   ThrowBinaryException(MissingDelegateError,FreeTypeLibraryIsNotAvailable,
-    draw_info->font)
+                       draw_info->font);
+  ARG_NOT_USED(encoding);
+  ARG_NOT_USED(offset);
+  ARG_NOT_USED(metrics);
 }
 #endif
 
@@ -1372,34 +1626,46 @@ static unsigned int RenderFreetype(Image *image,const DrawInfo *draw_info,
 %
 */
 
-static char *EscapeParenthesis(const char *text)
+static char *EscapeParenthesis(const char *source)
 {
   char
-    *buffer;
+    *destination;
 
   register char
+    *q;
+
+  register const char
     *p;
 
-  register long
-    i;
+  size_t
+    length;
 
-  unsigned long
-    escapes;
+  assert(source != (const char *) NULL);
 
-  escapes=0;
-  buffer=AllocateString(text);
-  p=buffer;
-  for (i=0; i < (long) Min(strlen(text),(MaxTextExtent-escapes-1)); i++)
-  {
-    if ((text[i] == '(') || (text[i] == ')'))
-      {
-        *p++='\\';
-        escapes++;
-      }
-    *p++=text[i];
-  }
-  *p='\0';
-  return(buffer);
+  /*
+    Use dry-run method to compute required string length.
+  */
+  length=0;
+  for (p=source; *p; p++)
+    {
+      if ((*p == '(') || (*p == ')'))
+        length++;
+      length++;
+    }
+  destination=MagickAllocateMemory(char *,length+1);
+  if (destination == (char *) NULL)
+    MagickFatalError3(ResourceLimitFatalError,MemoryAllocationFailed,
+      UnableToEscapeString);
+  *destination='\0';
+  q=destination;
+  for (p=source; *p; p++)
+    {
+      if ((*p == '(') || (*p == ')'))
+        *q++= '\\';
+      *q++=(*p);
+    }
+  *q=0;
+  return(destination);
 }
 
 static MagickPassFail RenderPostscript(Image *image,const DrawInfo *draw_info,
@@ -1672,7 +1938,7 @@ static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
 
   if (display == (Display *) NULL)
     {
-      char
+      const char
         *client_name;
 
       /*
@@ -1686,7 +1952,7 @@ static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
         Get user defaults from X resource database.
       */
       (void) XSetErrorHandler(MagickXError);
-      client_name=SetClientName((char *) NULL);
+      client_name=GetClientName();
       resource_database=MagickXGetResourceDatabase(display,client_name);
       MagickXGetResourceInfo(resource_database,client_name,&resource_info);
       resource_info.close_server=False;
@@ -1721,15 +1987,6 @@ static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
       font_info=MagickXBestFont(display,&resource_info,False);
       if (font_info == (XFontStruct *) NULL)
         ThrowBinaryException(XServerError,UnableToLoadFont,draw_info->font);
-      if ((map_info == (XStandardColormap *) NULL) ||
-          (visual_info == (XVisualInfo *) NULL) ||
-          (font_info == (XFontStruct *) NULL))
-        {
-          MagickXFreeResources(display,visual_info,map_info,&pixel,font_info,
-            &resource_info,(MagickXWindowInfo *) NULL);
-          ThrowBinaryException(XServerError,UnableToLoadFont,
-            draw_info->server_name)
-        }
       cache_info=(*draw_info);
     }
   /*
@@ -1800,8 +2057,10 @@ static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
 }
 #else
 static MagickPassFail RenderX11(Image *image,const DrawInfo *draw_info,
-  const PointInfo *ARGUNUSED(offset),TypeMetric *ARGUNUSED(metrics))
+  const PointInfo *offset,TypeMetric *metrics)
 {
+  ARG_NOT_USED(offset);
+  ARG_NOT_USED(metrics);
   ThrowBinaryException(MissingDelegateError,XWindowLibraryIsNotAvailable,
     draw_info->font);
 }
